@@ -4,21 +4,10 @@ use warnings;
 use base qw{Pod::Parser};
 use Hash::Merge qw(merge);
 
-# Set our version
-our $VERSION = '0.011';
-
-=pod
-
-=head1 VERSION
-
-0.011
-
-=cut
-
 # Hold data for our pending statement
 my $info = {};
 
-# Hold our SQL
+# SQL instance
 my %sql;
 
 # What command we're looking at
@@ -43,7 +32,6 @@ sub new {
     my ($class, $file, %arg) = @_;
     $scope = 'new';
     my %back = %sql;
-    #~ my %back_tt = %tt;
     
     $tt = $arg{TT} || $arg{tt} || {};
     $template = $arg{template} || {};
@@ -53,22 +41,22 @@ sub new {
     my $new = { %sql };
     
     %sql = %back;
-    #~ %tt = %back_tt;
+
     bless $new, $class;
 }
 
 # Taken directly from Class::Singleton
-sub instance000 {
-    my $class = shift;
-    $scope = 'instance';
-    # get a reference to the _instance variable in the $class package 
-    no strict 'refs';
-    my $instance = \${ "$class\::_instance" };
+#~ sub instance000 {
+    #~ my $class = shift;
+    #~ $scope = 'instance';
+    #~ # get a reference to the _instance variable in the $class package 
+    #~ no strict 'refs';
+    #~ my $instance = \${ "$class\::_instance" };
 
-    defined $$instance
-        ? $$instance
-        : ($$instance = $class->_instance(@_));
-}
+    #~ defined $$instance
+        #~ ? $$instance
+        #~ : ($$instance = $class->_instance(@_));
+#~ }
 
 my $instance;
 sub instance {
@@ -78,7 +66,7 @@ sub instance {
     my $tt = $arg{TT} || $arg{tt};
     @tt{ keys %$tt } = values %$tt
         if $tt;
-    #~ @template{ keys %{$arg{template}} } = values %{$arg{template}}
+    #~ @template{ keys %{$arg{template}} } = values %{$arg{template}} хэш сложный!
    
     %template = %{merge($arg{template}, \%template)}
         if $arg{template} && %{$arg{template}};
@@ -93,6 +81,7 @@ sub _process {# pos file
     $file .='.pm'
         if $file =~ s/::/\//g;
     #~ warn "Processing file [$file]";
+    $enc = undef;
     $class->SUPER::new->parse_from_file($file);
 }
 
@@ -111,7 +100,6 @@ sub command {
 
     # Get rid of all trailing whitespace
     $paragraph =~ s/\s+$//ms;
-    $enc && ($paragraph = Encode::decode($enc, $paragraph));
 
     # There may be a short description right after the command
     if ($command eq 'desc') {
@@ -131,7 +119,6 @@ sub command {
     
     if ($command eq 'encoding') {
         $enc = $paragraph;
-        require Encode;
     }
 
     # Remember what command we're in
@@ -150,8 +137,8 @@ sub end_input {
                 $info,
                 tt => {%TT, $scope eq 'new' ? %$tt : %tt},
                 template => $scope eq 'new' ? $template : \%template,
+                enc=>$enc,
             );
-            $sql{$info->{name}}->_eval_param() if $sql{$info->{name}}->param;
             # Start with a new empty hashref
             $info = {};
         } else {# Something's missing
@@ -167,7 +154,6 @@ sub textblock {
 
     # Collapse trailing whitespace to a \n
     $paragraph =~ s/\s+$/\n/ms;
-    $enc && ($paragraph = Encode::decode($enc, $paragraph));
 
     if ($cmd eq 'desc') {
         $info->{desc} .= $paragraph;
@@ -208,6 +194,7 @@ package DBIx::POS::Statement;
 #=============================================
 use Text::Template;
 use Hash::Merge qw(merge);
+use Encode;
 
 use overload '""' => sub { shift->template };
 
@@ -218,7 +205,13 @@ sub new {
     my %arg = @_;
     $self->{_TT} = $arg{TT} || $arg{tt} ;
     $self->{_template_default} = $arg{template};
+    $self->{_enc} = $arg{enc};
     bless ($self, $class);
+    if (my $enc = $self->{_enc}) {
+        my @enc = qw(name desc param short sql);
+        @$self{ @enc } = map Encode::decode($enc, $self->{$_}), @enc;
+    }
+    $self->_eval_param();
     return $self;
 }
 
@@ -246,13 +239,14 @@ sub param {# ->param() |  ->param('foo') | ->param('foo'=>'bar', ....)
     #~ $self->{param} ||= {};
     return $self->{param} unless ref $self->{param} eq 'HASH';
     return $self->{param} unless @_;
-    return $self->{param}{shift()} if @_ == 1;
+    return $self->{param}{ shift() } if @_ == 1;
     my %arg = @_;
     @{$self->{param} ||= {}}{ keys %arg } = values %arg;
 }
 
 sub _eval_param {
     my $self = shift;
+    return unless $self->{param};
     my $param = eval $self->{param};
     die "Malformed perl code param [$self->{param}]: $@" if $@;
     $self->{param} = $param;
@@ -288,8 +282,7 @@ sub template {
     $self->{_template}->fill_in(HASH=>merge(\%arg, $self->{_template_default}));
 }
 
-1;
-
+our $VERSION = '0.020';
 
 =pod
 
@@ -301,6 +294,10 @@ sub template {
 
 ¡ ¡ ¡ ALL GLORY TO GLORIA ! ! !
 
+=head1 VERSION
+
+0.020
+
 =head1 NAME
 
 DBIx::POS::Template - is a fork of L<DBIx::POS>. Define a dictionary of SQL statements in a POD dialect (POS) plus expand template sql with embedded Perl using L<Text::Template>.
